/******************************************************************************
 *
 * Copyright:
 *    (C) 2006 Audrius Urmanavicius
 *
 * File:
 *    color_edit.c
 *
 * Description:
 *    Implements color editor
 *
 *****************************************************************************/
#include "configAppl.h"
#ifdef INCLUDE_COLOR_EDITOR
/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include <string.h>
//#include "color_editor.h"
#include "lcd.h"
#include "key.h"
#include "arrow_img.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define ARROW_LEFT_BLANK	0x01
#define ARROW_LEFT_OPAQUE	0x03
#define ARROW_LEFT_TRANSP	0x07

#define ARROW_RIGHT_BLANK	0x08
#define ARROW_RIGHT_OPAQUE	0x18
#define ARROW_RIGHT_TRANSP	0x38


#define COLOR_MAX			7
#define BAR_STEP			14
#define BAR_LEFT			16

/******************************************************************************
 * Global variables
 *****************************************************************************/

/******************************************************************************
 * Local function forward definitions
 *****************************************************************************/
static void drawElements(const char *colName);

/*****************************************************************************
 *
 * Description:
 *    Draw static color editor elements
 *
 ****************************************************************************/
void drawElements(const char *colName) {
//	int i;
	lcdColor(clBlack, clBlack);
	lcdClrscr();
	
	// Caption
	lcdColor(clGray, clRed);
	lcdRect(0, 0, 130, 18, clGray);
	lcdGotoxy(65 - strlen(colName) * 4, 1);
	lcdPuts((tU8*)colName);

    // Color sample rectangle
	lcdRect(40, 22,  50, 50, clWhite);
	//Horizontal separator
	lcdRect( 0, 76, 130,  1, clWhite);

	// RGB bars
	lcdRect(BAR_LEFT - 1,  82, BAR_STEP * COLOR_MAX + 2, 10, 0xE9);
	lcdRect(BAR_LEFT - 1,  98, BAR_STEP * COLOR_MAX + 2, 10, 0x7D);
	lcdRect(BAR_LEFT - 1, 114, BAR_STEP * COLOR_MAX + 2, 10, 0x6B);
}

/*****************************************************************************
 *
 * Description:
 *    Draw color bar
 *
 * Paramters:
 *    bar: Bar number (0: Red, 1: Green, 2: Blue)
 *    val: bar value (range 0-7)
 *
 ****************************************************************************/
void drawBarValue(tU8 bar, tU8 val) {
	tU8 w = val * BAR_STEP;
	static tU8 rgb[] = {clRed, clGreen, clBlue};
	lcdRect(BAR_LEFT, 83 + (bar * 16), w, 8, rgb[bar]);
	lcdRect(BAR_LEFT + w, 83 + (bar * 16), BAR_STEP * COLOR_MAX - w, 8, clBlack);
}

/*****************************************************************************
 *
 * Description:
 *    Draw/remove arrows next to RGB bars
 *
 * Paramters:
 *    bar: Bar number (0: Red, 1: Green, 2: Blue)
 *    opts: arrow options, bit field:
 *         bit 0:		if 1, update left arrow, otherwise leave as is
 *         bits [2:1]:	if 00 - remove arrow
 *						if 01 - draw opaque arrow
 *						if 1x - draw transparent arrow
 *         bit 3:		if 1, update right arrow, otherwise leave as is
 *         bits [5:4]:	Right arrow flags. See description of bits 2:1
 *
 *    Note: no range checking performed
 *
 ****************************************************************************/
void drawBarArrows(tU8 bar, tU8 opts) {
	tU8 top = 78 + bar * 16;
	if(opts & 0x01) {
		if(opts & 0x06)
			lcdIcon(BAR_LEFT - 14, top, 10, 18, TRUE, 0x01, (opts & 0x04)? ImgLeftTransparent+4: ImgLeftOpaque+4);
		else
			lcdRect(BAR_LEFT - 14, top, 10, 18, clBlack);
	}
	if(opts & 0x08) {
		if(opts & 0x30)
			lcdIcon(BAR_LEFT + BAR_STEP * COLOR_MAX + 4, top, 10, 18, TRUE, 0x01, (opts & 0x20)? ImgRightTransparent+4: ImgRightOpaque+4);
		else
			lcdRect(BAR_LEFT + BAR_STEP * COLOR_MAX + 4, top, 10, 18, clBlack);
	}
}

/*****************************************************************************
 *
 * Description:
 *    Draw sample color rectangle
 *
 * Paramters:
 *    color: rectangle color
 *    label_col: color of index text inside rectangle
 *
 ****************************************************************************/
static void drawColorRect(tU8 color, tU8 label_col) {
	lcdRect(41, 23, 48, 48, color);
	lcdColor(color, label_col);
	lcdGotoxy(57, 40);
	lcdPutchar((color >> 4) + (((color >> 4) >= 10)? 'A' - 10: '0'));
	lcdPutchar((color & 0x0F) + (((color & 0x0F) >= 10)? 'A' - 10: '0'));
}

/*****************************************************************************
 *
 * Description:
 *    Color edit routine
 *
 ****************************************************************************/
void editColor(tU8 *col, const char*name) {
	tU8 key, activeBar;
	tU8 rgb[3];
	tU8 tmpCol;		
	drawElements(name);

	
    rgb[0] = (*col >> 5) & 0x07;
    rgb[1] = (*col >> 2) & 0x07;
    rgb[2] = (*col << 1) & 0x07;

	//Adjust Blue component to max, since Blue component is only 2 bits wide
	if(rgb[2] == 6)
		rgb[2] = 7;


	for(activeBar = 0; activeBar < 3; activeBar++)
		drawBarValue(activeBar, rgb[activeBar]);

	activeBar = 0;
	drawBarArrows(activeBar, ((rgb[0] > 0)? ARROW_LEFT_OPAQUE: ARROW_LEFT_TRANSP) |
						  ((rgb[0] < 0x07)? ARROW_RIGHT_OPAQUE: ARROW_RIGHT_TRANSP));
	do {
	    drawColorRect(*col, (rgb[0] + rgb[1] + rgb[2] > 10)? clBlack: clWhite);
		drawBarValue(activeBar, rgb[activeBar]);
		key = getKey();

		if(key & (KEY_UP | KEY_DOWN)) {
			drawBarArrows(activeBar, ARROW_LEFT_BLANK | ARROW_RIGHT_BLANK);
			if(key == KEY_UP) {
				if(activeBar > 0)
					activeBar--;
				else
					activeBar = 2;
			} else {
				if(activeBar < 2)
					activeBar++;
				else
					activeBar = 0;
			}
			drawBarArrows(activeBar, ((rgb[activeBar] > 0)? ARROW_LEFT_OPAQUE: ARROW_LEFT_TRANSP) |
								  ((rgb[activeBar] < 0x07)? ARROW_RIGHT_OPAQUE: ARROW_RIGHT_TRANSP));
		}

        if(key & (KEY_LEFT | KEY_RIGHT)) {
			if((key == KEY_LEFT) && (rgb[activeBar] > 0)) {
				if(rgb[activeBar] == 0x07)
					drawBarArrows(activeBar, ARROW_RIGHT_OPAQUE);
				rgb[activeBar]--;
				drawBarValue(activeBar, rgb[activeBar]);
				if(!rgb[activeBar])
					drawBarArrows(activeBar, ARROW_LEFT_TRANSP);
			} else if((key == KEY_RIGHT) && (rgb[activeBar] < 0x07)) {
				if(!rgb[activeBar])
					drawBarArrows(activeBar, ARROW_LEFT_OPAQUE);
				rgb[activeBar]++;
				drawBarValue(activeBar, rgb[activeBar]);
				if(rgb[activeBar] == 0x07)
					drawBarArrows(activeBar, ARROW_RIGHT_TRANSP);
			}
            tmpCol = ((rgb[0]  << 5) & 0xE0) | ((rgb[1] << 2) & 0x1C) | ((rgb[2] >> 1) & 0x03);
			if(tmpCol != *col) {
				*col = tmpCol;
				drawColorRect(*col, (rgb[0] + rgb[1] + rgb[2] > 10)? clBlack: clWhite);
			}
		}
	} while(key != KEY_CENTER);		
}
#endif /* INCLUDE_COLOR_EDITOR */
