/******************************************************************************
 *
 * Copyright:
 *    Audrius Urmanavicius (C) 2006
 *
 * File:
 *    fs_utils.c
 *
 * Description:
 *    File includes routines for non-volatile file system (NVFS) demonstration,
 *    testing and maintenance.
 *
 *****************************************************************************/
#include "configAppl.h"
#ifdef INCLUDE_FS_UTILS

//STD_IO_NAMING must be defined to be able to compile this file correctly.
#ifndef STD_IO_NAMING
	#error STD_IO_NAMING is not defined.
#endif

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include "../pre_emptive_os/api/general.h"
#include <string.h>
#include "hw.h"
#include "lcd.h"
#include "key.h"
#include "nvfs.h"
#include "eeprom.h"
#include "menu.h"

#include <stdio.h>
#include <stdlib.h>

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
typedef fsErrno (FileOperation)(FILE *f);

/*****************************************************************************
 * Local variables
 ****************************************************************************/
static const char *fname = "Test";
static const char *strDummy = "Just a dummy\nstring in a file.";

/*****************************************************************************
 * Forward definitions
 ****************************************************************************/
void fileOperation(tU32 op);
fsErrno writeFile(FILE *);
fsErrno readFile(FILE *);
fsErrno truncateFile(FILE *);
void dumpEEPROM(tU32);
void formatFS(tU32);

/*****************************************************************************
 * Menu definitions
 ****************************************************************************/
static tMenuItem const items[] = {
	{"Format FS", formatFS, 0},
	{"Write File", fileOperation, (tU32)&writeFile},
	{"Read File", fileOperation, (tU32)&readFile},
	{"Truncate File", fileOperation, (tU32)&truncateFile},
	{"EEPROM Dump", dumpEEPROM, 0},
//	{"Delete File", deleteFile, 0},
	{"Exit Utils", NULL, 0} };

static tMenu menu = {"FS Utils", NULL, items, sizeof(items) / sizeof(tMenuItem), 0};

/*****************************************************************************
 *
 * Description:
 *    dumps EEPROM
 *
 ****************************************************************************/
static void dumpEEPROM(tU32 dummy) {
	tU8 buff[8];
	tU8 p;
	tU16 i, j, start = 0;

	lcdColor(clBlack, clGreen);
	lcdClrscr();
	lcdPuts(" *** EEPROM *** \n");

	for(;;) {
		lcdGotoxy(0, 16);
	    for(i = 0; i < 8; i++) {
			eepromPageRead(start + i * 8, buff, sizeof(buff));
			for(j = 0; j < 8; j++) {
				lcdColor((i & 2)? clBlue: clBlack, (j & 1)? clWhite: clGray);
				p = (buff[j] >> 4) + '0';
				if(p > '9')
					p += 'A' - '9' - 1;
				lcdPutchar(p);
				p = (buff[j] &0x0F) + '0';
				if(p > '9')
					p += 'A' - '9' - 1;
				lcdPutchar(p);
			}
			lcdPutchar('\n');
		}

		switch(getKey()) {
			case KEY_UP:
					if(start)
						start -= 64;
					break;
			case KEY_DOWN:
					if(start < EEPROM_SIZE - 64)
						start += 64;
					break;
			case KEY_CENTER:	
				return;
		}
	}
}

/*****************************************************************************
 *
 * Description:
 *    Prints "Bad" and flashes Red LED 'err' times.
 *
 ****************************************************************************/
static void handleError(fsErrno err) {
	if(err != fsSuccess) {
		if(err == fsEOF) {
			lcdColor(clBlack, clYellow);
			lcdPuts("EOF\n");
		} else {
			lcdColor(clBlack, clRed);
			lcdPuts("Err\n");
			for(; err > 0; err--) {
				setLED(LED_RED, TRUE);
				osSleep(30);
				setLED(LED_RED, FALSE);
				osSleep(40);
			}
		}
    } else {
		lcdColor(clBlack, clGreen);
		lcdPuts("OK\n");
	}
	lcdColor(clBlack, clGray);
}

/*****************************************************************************
 *
 * Description:
 *    Generic file operation: opens, invokes file operation, closes file
 *    and handles errors.
 *
 ****************************************************************************/
void fileOperation(tU32 op) {
	FILE *file;
	FileOperation *fileOp = (FileOperation*)op;
	fsErrno err;

	lcdColor(clBlack, clGray);
	lcdClrscr();
		
	lcdPuts("Opening file-");
	file = fopen(fname, NULL);
	if(!file) {
		handleError(_fsErrno);
	} else {
		handleError(fsSuccess);
		err = fileOp(file);
		if(err)
			handleError(err);
		lcdPuts("Closing file-");
		if(fclose(file) != 0)
			handleError(ferror(file));
		else
			handleError(fsSuccess);
	}
	while(getKey() != KEY_CENTER);
}			

/*****************************************************************************
 *
 * Description:
 *    Writes some dummy string to file.
 *
 ****************************************************************************/
fsErrno writeFile(FILE *file) {
	lcdPuts("Write string-");
	fwrite(strDummy, 1, strlen(strDummy) + 1, file);
	if(ferror(file) != 0)
		return ferror(file);
	handleError(fsSuccess);
	return fsSuccess;
}
/*****************************************************************************
 *
 * Description:
 *    Reads a file and displays its contents
 *
 ****************************************************************************/
fsErrno readFile(FILE *file) {
	size_t n;
	tU8 buffer[32];

	memset(buffer, 0, sizeof(buffer));

	lcdPuts("Reading-");
	n = fread(buffer, 20, 1, file);
	if(!n && (ferror(file) != fsSuccess))
		return ferror(file);
	handleError(fsSuccess);
	lcdPuts("Contents:\n");
	lcdColor(clBlue, clYellow);
	lcdPuts(buffer);
	lcdColor(clBlack, clGray);
	memset(buffer, 0, sizeof(buffer));
	lcdPuts("\nRead further-");
	n = fread(buffer, 12, 1, file);
	if(!n && (ferror(file) != fsSuccess))
		return ferror(file);

	handleError(fsSuccess);
	lcdPuts("Contents:\n");
	lcdColor(clBlue, clYellow);
	lcdPuts(buffer);
	lcdColor(clBlack, clGray);
	lcdPutchar('\n');
	return fsSuccess;
}
/*****************************************************************************
 *
 * Description:
 *    Truncates a file after 1st sector
 *
 ****************************************************************************/
fsErrno truncateFile(FILE *file) {
	lcdPuts("Seeking-");
	if(fseek(file, 10, SEEK_SET) != 0)
		return ferror(file);

	handleError(fsSuccess);
	lcdPuts("Truncating-");
	if(ftrunc(file) != 0)
		return ferror(file);

	handleError(fsSuccess);
	return fsSuccess;
}

/*****************************************************************************
 *
 * Description:
 *    Initializes FileSystem
 *
 ****************************************************************************/
static void formatFS(tU32 dummy) {
	lcdColor(clBlack, clGray);
	lcdClrscr();
	lcdPuts("FS format-");
	fsFormat();
	handleError(_fsErrno);
	while(getKey() != KEY_CENTER);
}

/*****************************************************************************
 *
 * Description:
 *    Runs FS Utils application
 *
 ****************************************************************************/
void runFS_Utils(tU32 dummy) {
	tU8 i;
    menu.initialChoice = 0;
    do {
        i = runMenu(&menu);
    } while(i < menu.itemCount - 1);
}
#endif /* INCLUDE_FS_UTILS */
