/******************************************************************************
 *
 * Copyright:
 *    (C) 2006 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Main function of the "LPC2104 Color LCD Game Board with Bluetooth"
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include "../pre_emptive_os/api/general.h"
#include <printf_P.h>
#include <ea_init.h>
#include <stdlib.h>

#include "lcd.h"
#include "key.h"
#include "uart.h"
#include "menu.h"
#include "exampleGame.h"
#include "snake.h"
#include "pong.h"
#include "maze.h"
#include "bt.h"
#include "hw.h"
#include "version.h"
#include "configAppl.h"
#include "startupDisplay.h"
#include "customizer.h"
#include "fs_utils.h"
#include "nvfs.h"

#ifdef INCLUDE_MENU_FIRE
#include "fire_0_100x40c.h"
#include "fire_1_100x40c.h"
#include "fire_2_100x40c.h"
#include "fire_3_100x40c.h"
#include "fire_4_100x40c.h"
#include "fire_5_100x40c.h"
#include "fire_6_100x40c.h"
#endif

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define PROC1_STACK_SIZE 800
#define INIT_STACK_SIZE  600


/*****************************************************************************
 * Global variables
 ****************************************************************************/
volatile tU32 ms;

/*****************************************************************************
 * Local variables
 ****************************************************************************/
static tU8 proc1Stack[PROC1_STACK_SIZE];
static tU8 initStack[INIT_STACK_SIZE];
static tU8 pid1;

/*****************************************************************************
 * Local prototypes
 ****************************************************************************/
static void proc1(void* arg);
static void initProc(void* arg);

/*****************************************************************************
 * Menu callbacks/wrappers (mind the '_' suffix)
 ****************************************************************************/
#ifdef INCLUDE_EXAMPLE_GAME
void playTetris_(tU32 dummy) {playExample();};
#endif

#ifdef INCLUDE_SNAKE_GAME
void playSnake_(tU32 dummy) {playSnake();};
#endif

#ifdef INCLUDE_PONG_GAME
void playPong_(tU32 dummy) {playPong();};
#endif

#ifdef INCLUDE_BLUETOOTH
void handleBt_(tU32 dummy) {handleBt();};
#endif

/*****************************************************************************
 * Menu definitions
 ****************************************************************************/
static tMenuItem const items[] = {
#ifdef INCLUDE_EXAMPLE_GAME
	{"Play Tetris",	playTetris_,	0},
#endif
#ifdef INCLUDE_SNAKE_GAME
	{"Play Snake",		playSnake_,	0},
#endif
#ifdef INCLUDE_PONG_GAME
	{"Play Ping-Pong",	playPong_,	0},
#endif
#ifdef INCLUDE_MAZE_GAME
	{"Play Maze",		playMaze,	0},
#endif
#ifdef INCLUDE_FS_UTILS
	{"FileSys Utils",	runFS_Utils,	0},
#endif
#ifdef INCLUDE_BLUETOOTH
	{"Bluetooth",		handleBt_,	0},
#endif
#ifdef INCLUDE_CUSTOMIZER
	{"Customizer",	runCustomizer,	0},
#endif
 };

static tMenu menu = {"MENU", NULL, items, sizeof(items) / sizeof(tMenuItem), 0};
/*****************************************************************************
 *
 * Description:
 *    The first function to execute
 *
 ****************************************************************************/
int
main(void)
{
  tU8 error;
  tU8 pid;

  osInit();

  //immediate initilaizeation of hardware I/O pins
  immediateIoInit();

  osCreateProcess(initProc, initStack, INIT_STACK_SIZE, &pid, 1, NULL, &error);
  osStartProcess(pid, &error);

  osStart();
  return 0;
}

/*****************************************************************************
 *
 * Description:
 *    A process entry function
 *
 * Params:
 *    [in] arg - This parameter is not used in this application.
 *
 ****************************************************************************/
static void
proc1(void* arg)
{
#ifdef INCLUDE_STARTUP_SEQUENCE
  //shortly bleep with the buzzer and flash with the LEDs
  tU8 i,j;

  for(i=0; i<3; i++) {
    for(j=0; j<3; j++) {
      setBuzzer(TRUE);
      setLED(LED_GREEN, FALSE);
      setLED(LED_RED,   TRUE);
      osSleep(1);

      setBuzzer(FALSE);
      setLED(LED_GREEN, TRUE);
      setLED(LED_RED,   FALSE);
      osSleep(1);
    }
    osSleep(10);
  }
  setLED(LED_GREEN, FALSE);
  setLED(LED_RED,   FALSE);

#endif

  //display startup message
  resetLCD();
  lcdInit();

#ifdef INCLUDE_CUSTOMIZER
  runCustomizer(CUSTOMIZER_LOAD_SETTINGS);
#endif

#ifdef INCLUDE_STARTUP_SEQUENCE
  displayStartupSequence();
#endif

  // endless menu loop
    for(;;) {
        lcdColor(clBlack, clBlack);
        lcdClrscr();
        runMenu(&menu);
    }

/*#ifdef INCLUDE_MENU_FIRE
    switch(i)
    {
      case 0: lcdIcon(15, 88, 100, 40, _fire_0_100x40c[2], _fire_0_100x40c[3], &_fire_0_100x40c[4]); i++; break;
      case 1: lcdIcon(15, 88, 100, 40, _fire_1_100x40c[2], _fire_1_100x40c[3], &_fire_1_100x40c[4]); i++; break;
      case 2: lcdIcon(15, 88, 100, 40, _fire_2_100x40c[2], _fire_2_100x40c[3], &_fire_2_100x40c[4]); i++; break;
      case 3: lcdIcon(15, 88, 100, 40, _fire_3_100x40c[2], _fire_3_100x40c[3], &_fire_3_100x40c[4]); i++; break;
      case 4: lcdIcon(15, 88, 100, 40, _fire_4_100x40c[2], _fire_4_100x40c[3], &_fire_4_100x40c[4]); i++; break;
      case 5: lcdIcon(15, 88, 100, 40, _fire_5_100x40c[2], _fire_5_100x40c[3], &_fire_5_100x40c[4]); i++; break;
      case 6: lcdIcon(15, 88, 100, 40, _fire_6_100x40c[2], _fire_6_100x40c[3], &_fire_6_100x40c[4]); i = 0; break;
      default: i = 0; break;
    }
#endif
    osSleep(20);
  }          */
}


/*****************************************************************************
 *
 * Description:
 *    The entry function for the initialization process.
 *
 * Params:
 *    [in] arg - This parameter is not used in this application.
 *
 ****************************************************************************/
static void
initProc(void* arg) {
  tU8 error;

  eaInit();
#ifdef INCLUDE_NVFS
  fsInit();
#endif
  printf("\n*********************************************************");
  printf("\n*                                                       *");
  printf("\n* Welcome to Embedded Artists' summer promotion board;  *");
  printf("\n*   'LPC2104 Color LCD Game Board with Bluetooth'       *");
  printf("\n* in cooperation with Future Electronics and Philips.   *");
  printf("\n* Boards with embedded JTAG includes J-link(tm)         *");
  printf("\n* technology from Segger.                               *");
  printf("\n*                                                       *");

  printf("\n* Program version:  %d.%d                                 *", MAJOR_VER, MINOR_VER);
  printf("\n* Program date:     %s                          *", RELEASE_DATE);

  if (TRUE == ver1_0)
    printf("\n* Hardware version: 1.0                                 *");
  else if (TRUE == ver1_1)
    printf("\n* Hardware version: 1.1                                 *");

#ifdef __IAR_SYSTEMS_ICC__
  printf("\n* Compiled with IAR Embedded Workbench                  *");
#else
  printf("\n* Compiled with GCC                                     *");
#endif

  printf("\n*                                                       *");
  printf("\n* (C) Embedded Artists AB, 2006                         *");
  printf("\n*                                                       *");
  printf("\n*********************************************************\n");


  osCreateProcess(proc1, proc1Stack, PROC1_STACK_SIZE, &pid1, 3, NULL, &error);
  osStartProcess(pid1, &error);

  osInitTimers(&error);

#ifdef INCLUDE_BLUETOOTH
  initBtProc();
#endif

  initKeyProc();

  osDeleteProcess();
}

/*****************************************************************************
 *
 * Description:
 *    The timer tick entry function that is called once every timer tick
 *    interrupt in the RTOS. Observe that any processing in this
 *    function must be kept as short as possible since this function
 *    execute in interrupt context.
 *
 * Params:
 *    [in] elapsedTime - The number of elapsed milliseconds since last call.
 *
 ****************************************************************************/
void
appTick(tU32 elapsedTime) {
  ms += elapsedTime;
}
