/******************************************************************************
 *
 * Copyright:
 *    (C) 2006 Audrius Urmanavicius
 *
 * File:
 *    maze.c
 *
 * Description:
 *    Implements the maze game generation & play.
 *
 *****************************************************************************/
#include "ConfigAppl.h"
#ifdef INCLUDE_MAZE_GAME
/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include "../pre_emptive_os/api/general.h"
#include <ea_init.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "lcd.h"
#include "key.h"
#include "menu.h"
#include "hw.h"
#include "ConfigAppl.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

//Color setup
#define COLOR_BKGND             0x03
#define COLOR_BORDER            0x34
#define COLOR_INVISIBLE_WALL	0x49
#define COLOR_VISIBLE_WALL      0xFF
#define COLOR_VISITED           0x4B
#define COLOR_CURSOR            0x1C
#define COLOR_FINISH            0xE4

#define FIELD_WIDTH		120	// Game field width in px
#define FIELD_HEIGHT	100 // Game field height in px

#define MAXWIDTH        30	// Maze max width in cells
#define MAXHEIGHT       25    // Maze max height in cells

#define DIR_LEFT	    1	// Maze directions
#define DIR_RIGHT       2
#define DIR_UP		    3
#define DIR_DOWN	    4
#define DIR_INIT	    5  // Special maze direction used to mark initial location of maze generation

/*****************************************************************************
 * Local prototypes
 ****************************************************************************/
static void setupMaze(void);
static void drawMaze(void);
static void startMaze(tU32 gridSize);
static void quitMaze(tU32);
static void showHideWalls(tU32);
static tS8 checkWall(tS32 x, tS32 y, tS8 dir);
static void drawWall(tS32 x, tS32 y, tS8 dir, tU8 color);
static void drawCursor(void);
static void drawFinish(void);
static void showScore(void);
static void showCongratulations(void);

/*****************************************************************************
 * Local variables
 ****************************************************************************/
static tS8  mMaze[MAXHEIGHT][MAXWIDTH];
static tS32	width, height, ox, oy;
static tS32 currX, currY, lastX, lastY;
static tS32 cellWidth, cellHeight;
static tU32 wallHitCount, moveCount;
static tU8  showWalls, quitGame, finishGame;

/*****************************************************************************
 * Menu definitions
 ****************************************************************************/
static tMenuItem const items[] = {{"6 x 5",   startMaze, ( 6 + ( 5 << 8))},
                                  {"12 x 10", startMaze, (12 + (10 << 8))},
                                  {"24 x 20", startMaze, (24 + (20 << 8))},
                                  {"30 x 25", startMaze, (30 + (25 << 8))},
                                  {"Show walls", showHideWalls, NULL},
                                  {"Exit game", quitMaze, NULL} };

static tMenu menu = {"M-A-Z-E!", NULL, items, sizeof(items) / sizeof(tMenuItem), 0};

/*****************************************************************************
 * External variables
 ****************************************************************************/
extern volatile tU32 ms;

/*****************************************************************************
 *
 * Description:
 *    return cell value or -1 of request out of bounds
 *
 ****************************************************************************/
static tS8 getCell(tS32 x, tS32 y) {
    if((x >= 0) && (y >= 0) && (x < width) && (y < height))
        return mMaze[y][x];
    return -1;
}
/*****************************************************************************
 *
 * Description:
 *    run menu loop
 *
 ****************************************************************************/
void playMaze(tU32 dummy) {
    quitGame = FALSE;
    menu.initialChoice = 0;
	showWalls = TRUE;
	lcdColor(clBlack, clGray);
	lcdClrscr();	
    do {
        runMenu(&menu);
    } while(!quitGame);
}

/*****************************************************************************
 *
 * Description:
 *    Quit game (menu callback)
 *
 ****************************************************************************/
static void quitMaze(tU32 dummy) {
    quitGame = TRUE;
}

/*****************************************************************************
 *
 * Description:
 *    Show or hide walls in next puzzle (menu callback)
 *
 ****************************************************************************/
static void showHideWalls(tU32 dummy) {
	showWalls = !showWalls;
}

/*****************************************************************************
 *
 * Description:
 *    Implements Snake game. (menu callback)
 *
 ****************************************************************************/
void startMaze(tU32 gridSize) {
    tU8 keypress, dir;

    width =  gridSize & 0xFF;
    height = (gridSize >> 8) & 0xFF;

    cellWidth = FIELD_WIDTH / width;
    cellHeight = FIELD_HEIGHT / height;
    ox = oy = 4;

    srand(ms);
    setupMaze();
    drawMaze();

    currX = 0;  currY = 0;
    lastX = 0;  lastY = 0;
	wallHitCount = 0;
	moveCount = 0;
	finishGame = 0;

	drawFinish();
    drawCursor();
	showScore();
    do { //Game loop
        keypress = getKey();
        switch(keypress) {
            case KEY_LEFT:
				dir = DIR_LEFT;
                currX--;
                break;

            case KEY_RIGHT:
				dir = DIR_RIGHT;
                currX++;
                break;

            case KEY_UP:
				dir = DIR_UP;
                currY--;
                break;

            case KEY_DOWN:
				dir = DIR_DOWN;
                currY++;
                break;

			default:
				dir = 0;
        }

		if(dir) {
			if(checkWall(lastX, lastY, dir)) {
				drawCursor();
		        lastX = currX, lastY = currY;
				moveCount++;
                if((currX == width - 1) && (currY == height - 1))  //is lower right corner reached
					finishGame = TRUE;
			} else {
				currX = lastX, currY = lastY;
		        drawWall(currX, currY, dir, COLOR_VISIBLE_WALL);
				wallHitCount++;
			}
			showScore();
		}
    } while(!finishGame && (keypress != KEY_CENTER));

	if(finishGame) {
		showCongratulations();
	}
	lcdColor(clBlack, clGray);
	lcdClrscr();
}

/*****************************************************************************
 *
 * Description:
 *    Initialize a maze.
 *
 ****************************************************************************/
void setupMaze(void) {
	tS32 x = width / 2, y = height / 2;
	tS32 availDirs[4], dirCount;
    tS8 dir;

	memset(mMaze, 0, sizeof(mMaze));
	mMaze[y][x] = DIR_INIT; //Mark start position
	do {
		dirCount = 0;
		if(getCell(x - 1, y) == 0)
			availDirs[dirCount++] = DIR_LEFT;
		if(getCell(x + 1, y) == 0)
			availDirs[dirCount++] = DIR_RIGHT;
		if(getCell(x, y - 1) == 0)
			availDirs[dirCount++] = DIR_UP;
		if(getCell(x, y + 1) == 0)
			availDirs[dirCount++] = DIR_DOWN;

		if(dirCount) { // if there's at least one way to unmarked cell - go there
			dir = availDirs[rand() % dirCount];
			switch(dir) {
				case DIR_LEFT:	x--; break;
				case DIR_RIGHT: x++; break;
				case DIR_UP:	y--; break;
				case DIR_DOWN:	y++; break;
			}
			mMaze[y][x] = dir; 	// Store direction from which we entered this cell
		} else { 				// no ways left to progress - retard one cell
			switch(mMaze[y][x]) {
				case DIR_LEFT:	x++; break; // got here from right (i.e. LEFT move taken to get there)
				case DIR_RIGHT: x--; break;
				case DIR_UP:	y++; break;
				case DIR_DOWN:	y--; break;
			}
		}
	} while(mMaze[y][x] != DIR_INIT); // if we did not return to start - repeat generating.
}

/*****************************************************************************
 *
 * Description:
 *    Check for ability to move to selected direction
 *
 ****************************************************************************/
tS8 checkWall(tS32 x, tS32 y, tS8 dir) {
    tS8 cell = getCell(x, y);
    if(cell < 0)
        return 0;
    switch(dir) {
        case DIR_LEFT:  return (cell == DIR_RIGHT)  || (getCell(x - 1, y) == DIR_LEFT);
        case DIR_RIGHT: return (cell == DIR_LEFT)   || (getCell(x + 1, y) == DIR_RIGHT);
        case DIR_UP:    return (cell == DIR_DOWN)   || (getCell(x, y - 1) == DIR_UP);
        case DIR_DOWN:  return (cell == DIR_UP)     || (getCell(x, y + 1) == DIR_DOWN);
    }
    return 0;
}

/*****************************************************************************
 *
 * Description:
 *    Draw a single wall.
 *
 ****************************************************************************/
void drawWall(tS32 x, tS32 y, tS8 dir, tU8 color) {
    tS32 cellX, cellY;
    cellX = ((dir == DIR_RIGHT)? (x + 1): x) * cellWidth  + ox;
    cellY = ((dir == DIR_DOWN)?  (y + 1): y) * cellHeight + oy;

    switch(dir) {
        case DIR_LEFT:
        case DIR_RIGHT:
            lcdRect(cellX, cellY, 1, cellHeight + 1, color);
            break;
        case DIR_UP:
        case DIR_DOWN:
            lcdRect(cellX, cellY, cellWidth + 1, 1, color);
            break;
    }
}

/*****************************************************************************
 *
 * Description:
 *    Draw a maze.
 *
 ****************************************************************************/
void drawMaze(void) {
    tS32 x, y;

  //clear screen
  lcdColor(0,0xe0);
  lcdClrscr();

  //draw game field rectangle
  lcdRectBrd(0, 0, 129, 110, COLOR_BKGND, COLOR_BORDER, COLOR_BORDER);

	if(showWalls)
		for(x = 0; x < width; x++)
    		for(y = 0; y < height; y++) {
        		if(y && (mMaze[y][x] != DIR_DOWN && mMaze[y - 1][x] != DIR_UP))
            		drawWall(x, y, DIR_UP, COLOR_INVISIBLE_WALL);

		        if(x && (mMaze[y][x] != DIR_RIGHT && mMaze[y][x - 1] != DIR_LEFT))
        		    drawWall(x, y, DIR_LEFT, COLOR_INVISIBLE_WALL);
	    }

    lcdRect(ox, oy, 1, FIELD_HEIGHT + 1, COLOR_VISIBLE_WALL);
    lcdRect(ox, oy, FIELD_WIDTH + 1, 1, COLOR_VISIBLE_WALL);
    lcdRect(ox + FIELD_WIDTH, oy, 1, FIELD_HEIGHT + 1, COLOR_VISIBLE_WALL);
    lcdRect(ox, oy + FIELD_HEIGHT, FIELD_WIDTH + 1, 1, COLOR_VISIBLE_WALL);

//  showScore();
}

/*****************************************************************************
 *
 * Description:
 *    Draw cursor in a current position and mark previous position as visited
 *
 ****************************************************************************/
void drawCursor(void) {
    tS32 cellX, cellY;
    if(lastX != currX || lastY != currY) {
        cellX = ox + lastX * cellWidth;
        cellY = oy + lastY * cellHeight;
        lcdRect(cellX + 1, cellY + 1, cellWidth - 1, cellHeight - 1, COLOR_VISITED);
    }
    if(currX >= 0 && currY >= 0) {
        cellX = ox + currX * cellWidth;
        cellY = oy + currY * cellHeight;
        if(cellWidth > 4)
            lcdRect(cellX + 2, cellY + 2, cellWidth - 3, cellHeight - 3, COLOR_CURSOR);
        else
            lcdRect(cellX + 1, cellY + 1, cellWidth - 1, cellHeight - 1, COLOR_CURSOR);
    }
}

/*****************************************************************************
 *
 * Description:
 *    Draw a Finish location (bottom rightmost cell)
 *
 ****************************************************************************/
void drawFinish(void) {
	tS8 i = cellWidth / 4;
    tS32 cellX, cellY;
    cellX = ox + (width  - 1) * cellWidth;
    cellY = oy + (height - 1) * cellHeight;
    lcdRect(cellX + i + 1, cellY + 1, cellWidth - 2 * i - 1, cellHeight - 1, COLOR_FINISH);
    lcdRect(cellX + 1, cellY + i + 1, cellWidth - 1, cellHeight - 2 * i - 1, COLOR_FINISH);
}

/*****************************************************************************
 *
 * Description:
 *    Draw current score
 *
 ****************************************************************************/
static void showScore(void) {
    static char str[13];
	tS32 len;

	len = sprintf(str, "M: %d", moveCount);
	lcdColor(clBlack, clYellow);
	lcdGotoxy(32 - len * 4, 111);
	lcdPuts((tU8 *)str);

	len = sprintf(str, "W: %d", wallHitCount);
	lcdColor(clBlack, clRed);
	lcdGotoxy(96 - len * 4, 111);
	lcdPuts((tU8 *)str);
}

/*****************************************************************************
 *
 * Description:
 *    Show congratulations
 *
 ****************************************************************************/
static void showCongratulations(void) {
	tU8 flash = FALSE;
	do {
		if(flash)
			lcdColor(clYellow, clRed);
		else
			lcdColor(clBlue, clWhite);
		lcdGotoxy(1, 50);
		lcdPuts("Congratulations!");
		lcdGotoxy(1, 64);
		lcdPuts("Press CENTER key");
		flash = !flash;

		setBuzzer(TRUE);
		osSleep(2);
		setBuzzer(FALSE);
		osSleep(40);
	} while(checkKey() != KEY_CENTER);
}

#endif /* INCLUDE_MAZE_GAME */



