/******************************************************************************
 *
 * Copyright:
 *    2006 Audrius Urmanavicius
 *         MediaWorks
 *
 * File:
 *    menu.c
 *
 * Description:
 *    The internals of improved menu with extended functionality of select.c
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include "../pre_emptive_os/api/general.h"
#include <ea_init.h>
#include <stdlib.h>
#include <string.h>
#include "lcd.h"
#include "key.h"
#include "menu.h"

/******************************************************************************
 * Defines
 *****************************************************************************/
#define MAX_VISIBLE_ITEMS   6
#define POS_BAR_WIDTH       4
#define CAPTION_HEIGHT      17
#define ITEM_HEIGHT         16

/*****************************************************************************
 * Global variables
 ****************************************************************************/
tMenuColors defMenuColors = {0xFF, 0x33, 0x48, 0xE0, 0x24, 0x91, 0x03, 0xF8};

/*****************************************************************************
 * Common menu definitions
 ****************************************************************************/

static tMenuItem const gameOverItems[] = {
	{"Restart game", NULL, NULL},
	{"End game", NULL, NULL}};

static tMenu gameOverMenuDef = {"Game over!", NULL, gameOverItems, sizeof(gameOverItems) / sizeof(tMenuItem), 0};
tMenu *gameOverMenu = &gameOverMenuDef;

/*****************************************************************************
 * Local variables
 ****************************************************************************/
static tMenu *menu;
static tMenuColors const *menuColors;
static tU8 cursor;
static tU8 xPos, yPos, menuWidth, itemWidth, menuHeight;
static tS8 topItem, visibleItems;

/*****************************************************************************
 *
 * Description:
 *    Draw a menu item
 *
 * Params:
 *    [in] item - item to draw
 *    [in] highlight - indicates item under cursor
 *
 ****************************************************************************/
static void
drawItem(tU8 item, tU8 highlight) {
    tU32 y = yPos + (item - topItem) * ITEM_HEIGHT + CAPTION_HEIGHT;
    if(highlight) {
        lcdColor(menuColors->cursorBg, menuColors->cursorFg);
        lcdRectBrd(xPos + 2, y, itemWidth - 2, ITEM_HEIGHT + 1,
            menuColors->cursorBg, menuColors->cursorBorder, menuColors->cursorBorder);
    } else {
        lcdColor(menuColors->itemBg, menuColors->itemFg);
        lcdRect(xPos + 1, y, itemWidth, ITEM_HEIGHT + 1, menuColors->itemBg);
    }
    lcdGotoxy(xPos + 5, y + 1);
    lcdPuts(menu->items[item].caption);
}

/*****************************************************************************
 *
 * Description:
 *    Draw a menu position bar (if there are more menu items than fits into screen)
 *
 * Params:
 *    [none]
 *
 ****************************************************************************/
static void
drawPosBar(void) {
    tU32 x, y, h;

    if(visibleItems == menu->itemCount)
        return;
    x = xPos + menuWidth - POS_BAR_WIDTH;
    y = yPos + CAPTION_HEIGHT;
    h = menuHeight - CAPTION_HEIGHT - 2;
    lcdRect(x, y, POS_BAR_WIDTH - 1, h, menuColors->itemBg);
    y += h * topItem / menu->itemCount;
    h = h * visibleItems / menu->itemCount;
    lcdRect(x, y, POS_BAR_WIDTH - 1, h, menuColors->border);
}

/*****************************************************************************
 *
 * Description:
 *    Draws/updates items and menu position bar
 *
 * Params:
 *    [none]
 *
 ****************************************************************************/
static void
drawItems(void) {
    tU8 i;
    for(i =  topItem; i < visibleItems  + topItem; i++)
        drawItem(i, cursor == i);

    drawPosBar();
}
/*****************************************************************************
 *
 * Description:
 *    Draw a menu
 *
 * Params:
 *    [none]
 *
 ****************************************************************************/
static void
drawMenu(void) {
    tU8 len;
    lcdRectBrd(xPos, yPos, menuWidth, menuHeight, menuColors->captionBg, menuColors->border, menuColors->border);
    lcdRect(xPos, yPos + CAPTION_HEIGHT - 1, menuWidth, 1, menuColors->border);
    if(visibleItems < menu->itemCount)
        lcdRect(xPos + menuWidth - POS_BAR_WIDTH - 1, yPos + CAPTION_HEIGHT,
                1, menuHeight - CAPTION_HEIGHT - 1, menuColors->border);

    len = strlen(menu->caption);
    lcdColor(menuColors->captionBg, menuColors->captionFg);
    lcdGotoxy(xPos + (menuWidth - len * 8) / 2, yPos + 1);
    lcdPuts(menu->caption);

    drawItems();
}

/*****************************************************************************
 *
 * Description:
 *    Displays and activates menu, invokes selected menu item callback
 *    if it is defined (i.e. callback != NULL) before returning.
 *
 ****************************************************************************/
tU8 runMenu(tMenu *m) {
    tU8 i, len, maxLen, key;
    tS8 oldTopItem, oldCursor;

    menu = m;
    menuColors = menu->colors;
    if(menuColors == NULL)
        menuColors = &defMenuColors;

    // find longest menu item caption and set appropriate menu width
    maxLen = strlen(menu->caption);
    for(i = 0; i < menu->itemCount; i++) {
        len = strlen(menu->items[i].caption);
        if(len > maxLen)
            maxLen = len;
    }

    itemWidth = 8 + 8 * maxLen;

    if(menu->itemCount <= MAX_VISIBLE_ITEMS) {
        visibleItems = menu->itemCount;
        menuWidth = itemWidth + 2;
     } else {
        visibleItems = MAX_VISIBLE_ITEMS;
        menuWidth = itemWidth + POS_BAR_WIDTH + 2;
    }

    menuHeight = ITEM_HEIGHT * visibleItems + CAPTION_HEIGHT + 3; //include 3 border lines
    xPos = (130 - menuWidth) / 2;
    yPos = (130 - menuHeight) / 2;
    cursor = (menu->initialChoice >= 0)? menu->initialChoice: 0;
    topItem = 0;
    if(cursor >= topItem + visibleItems - 1) {
        topItem = cursor - visibleItems + 2;
        if(topItem + visibleItems > menu->itemCount)
            topItem = menu->itemCount - visibleItems;
    }

    drawMenu();

    //reset any previous key stroke
    checkKey();

    for(;;) {
        key = getKey();

        if(key & (KEY_UP | KEY_DOWN | KEY_LEFT | KEY_RIGHT)) {
            oldTopItem = topItem;
            oldCursor = cursor;
            if((key & KEY_UP) && (cursor > 0))
                cursor--;
            if((key & KEY_DOWN) && (cursor < menu->itemCount - 1))
                cursor++;
            if(key & KEY_LEFT)
                cursor = 0;
            if(key & KEY_RIGHT)
                cursor = menu->itemCount - 1;

            if(cursor <= topItem) {
                topItem = (cursor > 0)? cursor - 1: 0;
            } else if(cursor >= topItem + visibleItems - 1) {
                topItem = cursor - visibleItems + 2;
                if(topItem + visibleItems > menu->itemCount)
                    topItem = menu->itemCount - visibleItems;
            }

            if(oldTopItem != topItem)
                drawItems();
            else if(oldCursor != cursor) {
                drawItem(oldCursor, FALSE);
                drawItem(cursor, TRUE);
            }

        } else if(key == KEY_CENTER) {
            tMenuItem const *item = &(menu->items[cursor]);
            if(menu->initialChoice >= 0)
                menu->initialChoice = cursor;

			i = cursor; //Preserve 'cursor' variable in case if callback proc calls another menu
            if(item->callback)
                item->callback(item->param);
            return i;
        }
    }
}
