/******************************************************************************
 *
 * Copyright:
 *    (C) 2006 Audrius Urmanavicius
 *
 * File:
 *    nvfs.c
 *
 * Description:
 *    Implements routines for basic file I/O (just fread(), fwrite() and some
 *    basic file manipulation routines, no fgetc/fgets/fputc/fputs/fprintf etc.)
 *
 *****************************************************************************/

#include "configAppl.h"
#ifdef INCLUDE_NVFS

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include <string.h>
#include "eeprom.h"
#include "nvfs.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

#define MAGIC_VALUE			0xF5
#define SECTOR_SIZE			16
#define CHAIN_TABLE_SIZE	(EEPROM_SIZE / SECTOR_SIZE)
#define SECTOR_COUNT		CHAIN_TABLE_SIZE

#define SECT_FREE		0x00
#define SECT_LAST		0x01
#define SECT_META		0x02
#define SECT_RESERVED	0x03


#define ERR_CHECK(a)	if((a)) return -1

typedef SectAddr fsSector[SECTOR_SIZE];

typedef struct {
	char name[6];
	tU8 zero;  // Used as 'name' terminator
	SectAddr firstSect;
} fsMetadata;

/******************************************************************************
 * Global variables
 *****************************************************************************/
static SectAddr chainTable[CHAIN_TABLE_SIZE];
static fsFile metaFile;
static const tU32 ZeroBlock[SECTOR_SIZE / sizeof(tU32)] = {0, 0, 0, 0};
static fsFile files[FOPEN_MAX];
fsErrno _fsErrno;

/******************************************************************************
 * Local function forward definitions
 *****************************************************************************/
static fsErrno storageWrite(tU16 addr, const void *data, const size_t size);
static fsErrno storageRead(tU16 addr, void *data, const size_t size);
#define getSectData(saddr)	(chainTable[saddr])
static fsErrno setSectData(SectAddr addr, SectAddr data);
static int fsAppendSector(fsFile *file);

/******************************************************************************
 * Implementation of public functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Sets current file read/write position and sets EOF flag
 *
 * Returns:
 *    0 on success, -1 on error
 *
 *****************************************************************************/
int fsSeek(fsFile *file, int pos, int origin) {
	SectAddr saddr;

	_fsErrno = fsSuccess;

// Some parameter sanity checks goes here
	if(!file || (file->state != FILE_OPEN)) {
		_fsErrno = fsInvalidArgument;
		return -1;
	}

	file->errno = fsSuccess;

    switch(origin) {
		case SEEK_CUR: pos += file->fpos; break;
		case SEEK_SET: break;
		case SEEK_END: //TODO: Implement seek to the end of file. NOW falls through to default:
		default: file->errno = fsInvalidArgument; return -1;
	}

	if(pos < 0) {
		file->errno = fsBadSeek;
		return -1;
	}

	file->EOF = FALSE;
	file->fpos = 0;

	saddr = file->firstSect;
	if(!saddr) {
		file->EOF = TRUE;
		if(pos != 0) {
			file->errno = fsBadSeek;
			return -1;
		} else
			return 0;
	}

/* Disable this optimisation for now
    if(file->fpos / SECTOR_SIZE == pos / SECTOR_SIZE) { //Check if the required sector is already loaded
		file->fpos = pos;
		return fsSuccess;
	} */

//	'saddr' will always initially contain a valid value, no need to doublecheck it
    while(pos >= SECTOR_SIZE) {
		saddr = getSectData(saddr);
		pos -= SECTOR_SIZE;
		file->fpos += SECTOR_SIZE;

		if(saddr <= SECT_RESERVED) {
			if(saddr == SECT_LAST) {
				file->EOF = TRUE;
				if(pos == 0)
					saddr = 0;
				else {
					file->currentSect = 0;
					file->errno = fsEOF;
					return -1;
				}
			} else {
				_fsErrno = fsFilesystemError;
				return -1;
			}
		}
	}	

	file->fpos += pos;
	file->currentSect = saddr;
	return 0;
}

/******************************************************************************
 *
 * Description:
 *    Reads 'size' bytes into 'ptr' buffer from currnet file position
 *
 * Returns:
 *    number of successfully read elements.
 *
 *****************************************************************************/
size_t fsRead(void *ptr, size_t size, size_t nelem, fsFile *file) {
	size_t ss = 0;
	size_t n = size * nelem;
//	fsErrno err;

// Some parameter sanity checks goes here
	if(!file || (file->state != FILE_OPEN) || !ptr) {
		_fsErrno = fsInvalidArgument;
		return 0;
	}

	file->errno = fsSuccess;

	while(n) {
		if(file->EOF) {
			file->errno = fsEOF;
			break;
		}

		ss = SECTOR_SIZE - (file->fpos & (SECTOR_SIZE - 1));
		if(ss > n)
			ss = n;

		if(storageRead(file->currentSect * SECTOR_SIZE + (file->fpos & (SECTOR_SIZE - 1)), ptr, ss) != 0) {
			file->errno = fsEepromError;
			break;
		}

		n -= ss;
		*((tU8 **)&ptr) += ss;

		if(fsSeek(file, ss, SEEK_CUR) != 0)
			break;
	}
	return nelem - n / size;
}

/******************************************************************************
 *
 * Description:
 *    Writes 'size' bytes from 'ptr' buffer into file at currnet file position.
 *    Allocates more space for file if necessary.
 *
 * Returns:
 *    Error code
 *
 *****************************************************************************/
size_t fsWrite(const void *ptr, size_t size, size_t nelem, fsFile *file) {
	size_t ss;
	size_t n = size * nelem;
//	fsErrno err;

// Some parameter sanity checks goes here
	if(!file || (file->state != FILE_OPEN) || !ptr) {
		_fsErrno = fsInvalidArgument;
		return 0;
	}

	file->errno = fsSuccess;
	while(n) {
		if(file->EOF) {
			if(fsAppendSector(file) != 0)
				break;
		}

		ss = SECTOR_SIZE - (file->fpos & (SECTOR_SIZE - 1));
		if(ss > n)
			ss = n;

		if(storageWrite(file->currentSect * SECTOR_SIZE + (file->fpos & (SECTOR_SIZE - 1)), ptr, ss) != fsSuccess) {
			file->errno = fsEepromError;
			break;
        }

		n -= ss;
		*((tU8 **)&ptr) += ss;

		if(fsSeek(file, ss, SEEK_CUR) != 0)
			break;
	}
	return nelem - n / size;
}

/******************************************************************************
 *
 * Description:
 *    Truncates file starting with the next sector of current file position.
 *
 * Returns:
 *    Error code
 *
 *****************************************************************************/
int fsTruncate(fsFile *file) {
	tU16 save_fpos;
	SectAddr saddr, next_saddr;

// Some parameter sanity checks goes here
	if(!file || (file->state != FILE_OPEN)) {
		_fsErrno = fsInvalidArgument;
		return -1;
	}
	file->errno = fsSuccess;

	if(file->EOF)
		return 0;

	save_fpos = file->fpos;

	if(save_fpos == 0) {  // Truncate whole file
		file->firstSect = 0;
		if(fsSeek(&metaFile, file->metadataOffset + sizeof(fsMetadata) - sizeof(SectAddr), SEEK_SET) != 0) {
			file->errno = fsFilesystemError;
			return -1;
		}
		fsWrite(&file->firstSect, sizeof(SectAddr), 1, &metaFile);
		if(metaFile.errno) {
			file->errno = fsFilesystemError;
			return -1;
		}
		return fsSeek(file, 0, SEEK_SET);
	}

	ERR_CHECK(fsSeek(file, -1, SEEK_CUR));
	saddr = getSectData(file->currentSect);
	setSectData(file->currentSect, SECT_LAST);

	while(saddr > SECT_RESERVED) {
		next_saddr = getSectData(saddr);
		setSectData(saddr, SECT_FREE);
		saddr = next_saddr;
	}

	return fsSeek(file, save_fpos, SEEK_SET);
}

/******************************************************************************
 *
 * Description:
 *    Opens (or creates, if does not exist) new file. File name must contain at
 *    least one character and no more than 6 total (not counting trailing '\0').
 *    There are no restrictions on characters in a file name; file names are
 *    case sensitive. Note that 'mode' parameter is ignored.
 *
 * Returns:
 *    pointer to fsFile structure.
 *
 *****************************************************************************/
fsFile * fsOpen(const char *name, const char *mode) {
	fsFile *file = NULL;
	fsMetadata metadata;
//    fsErrno err;
	tU16 last_fpos;
	int i;

	if((*name == 0) || (strlen(name) > sizeof(metadata.name))) {
		_fsErrno = fsInvalidName;
		return NULL;
	}

    for(i = 0; i < FOPEN_MAX; i++) {
		if(files[i].state == FILE_CLOSED) {
			file = &files[i];
			break;
		}
	}

	if(!file) {
		_fsErrno = fsTooManyOpenFiles;
		return NULL;
	}

    memset(file, 0, sizeof(fsFile));


	if(fsSeek(&metaFile, 0, SEEK_SET) != 0) {
		_fsErrno = metaFile.errno;
		return NULL;
	}

	for(;;) {
		last_fpos = metaFile.fpos;
		if(metaFile.EOF) {
			fsWrite(ZeroBlock, 1, SECTOR_SIZE, &metaFile);
			if(metaFile.errno) {
				_fsErrno = fsFilesystemError;
				return NULL;
			}
			memset(&metadata, 0, sizeof(metadata));
        } else {
			fsRead(&metadata, 1, sizeof(metadata), &metaFile);
			if(metaFile.errno) {
				_fsErrno = fsFilesystemError;
				return NULL;
			}
		}

		if(*metadata.name == 0) { // File not found - create one
			strncpy(metadata.name, name, sizeof(metadata.name));
			metadata.firstSect = 0;
			if(fsSeek(&metaFile, last_fpos, SEEK_SET) != 0) {
				_fsErrno = fsFilesystemError;
				return NULL;
			}

			fsWrite(&metadata, sizeof(metadata), 1, &metaFile);
			if(metaFile.errno) {
				_fsErrno = fsFilesystemError;
				return NULL;
			}

		}
		if(strcmp(name, metadata.name) == 0) {
			file->metadataOffset = last_fpos;
			file->firstSect = metadata.firstSect;
			file->state = FILE_OPEN;
			fsSeek(file, 0, SEEK_SET);
			return file;
		}
	}
}

/******************************************************************************
 *
 * Description:
 *    Closes previously open file. Be sure to close a file after use so it's
 *    fsFile structure can be reused in future fsOpen() calls.
 *
 * Returns:
 *    Error code
 *    Sets up 'file' variable.
 *
 *****************************************************************************/
int fsClose(fsFile *file) {
	if(!file || (file->state != FILE_OPEN)) {
		_fsErrno = fsInvalidArgument;
		return -1;
	}

	file->state = FILE_CLOSED;

	return 0;
}

/******************************************************************************
 *
 * Description:
 *    Initializes file system and global variables. Must be called only once
 *    and BEFORE any other fs*() calls.
 *
 * Returns:
 *    Error code
 *
 *****************************************************************************/
int fsInit(void) {
	eepromPageRead(0, chainTable, sizeof(chainTable));
	if(*(tU8*)chainTable != MAGIC_VALUE) {
		_fsErrno = fsFilesystemError;
		return -1;
	}

	memset(files, 0, sizeof(files));
	memset(&metaFile, 0, sizeof(metaFile));
	metaFile.firstSect = (CHAIN_TABLE_SIZE / SECTOR_SIZE);
	metaFile.currentSect = metaFile.firstSect;
	metaFile.state = FILE_OPEN;
	return 0;
}

/******************************************************************************
 *
 * Description:
 *    Formats file system.
 *
 * Returns:
 *    Error code
 *
 *****************************************************************************/
int fsFormat(void) {
	tU16 i;
	fsErrno err;

	i = MAGIC_VALUE;

	if((err = storageWrite(0, &i, sizeof(SectAddr))) != fsSuccess) {
		_fsErrno = err;
		return -1;
	}

	for(i = sizeof(SectAddr); i < CHAIN_TABLE_SIZE / SECTOR_SIZE; i++) {
		if((err = setSectData(i, SECT_RESERVED)) != fsSuccess) {
			_fsErrno = err;
			return -1;
		}
	}

	for(i = CHAIN_TABLE_SIZE / SECTOR_SIZE + 1; i < SECTOR_COUNT; i++) {
		if((err = setSectData(i, SECT_FREE)) != fsSuccess) {
			_fsErrno = err;
			return -1;
		}
	}

	//Mark first free sector as last metaFile sector
	if((err = setSectData(CHAIN_TABLE_SIZE / SECTOR_SIZE, SECT_LAST)) != fsSuccess) {
		_fsErrno = err;
		return -1;
	}

	//Zero out first metaFile sector
	storageWrite(CHAIN_TABLE_SIZE, ZeroBlock, sizeof(ZeroBlock));

	return fsInit();
}

/******************************************************************************
 * Implementation of local functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    "Device independent" write to an EEPROM.
 *
 * Returns:
 *    Error code
 *
 *****************************************************************************/
fsErrno storageWrite(tU16 addr, const void *data, const size_t size) {
	if(eepromWrite(addr, (tU8 *)data, size) != I2C_CODE_OK)
		return fsEepromError;
	if(eepromPoll() != I2C_CODE_OK)
		return fsEepromError;
	return fsSuccess;
}

/******************************************************************************
 *
 * Description:
 *    "Device independent" read from an EEPROM.
 *
 * Returns:
 *    Error code
 *
 *****************************************************************************/
fsErrno storageRead(tU16 addr, void *data, const size_t size) {
	if(eepromPageRead(addr, (tU8 *)data, size) != I2C_CODE_OK)
		return fsEepromError;
	return fsSuccess;
}

/******************************************************************************
 *
 * Description:
 *    Sets sector data (next sector in a chain or indicate last/special/free
 *    sector). Updates EEPROM as well as chain table cache in RAM.
 *
 * Returns:
 *    Error code
 *
 *****************************************************************************/
fsErrno setSectData(SectAddr addr, SectAddr data) {
	if(addr >= CHAIN_TABLE_SIZE)
		return fsInvalidAddress;
	
    chainTable[addr] = data;
	return storageWrite(addr * sizeof(SectAddr), &data, sizeof(SectAddr));
}

/******************************************************************************
 *
 * Description:
 *    Appends one sector to the end of file.
 *
 * Returns:
 *    0 on success, -1 on error
 *
 *****************************************************************************/
int fsAppendSector(fsFile *file) {
	SectAddr saddr = 2, new_saddr;
//	fsErrno err;

	// Find frst free sector
	do {
		if(getSectData(saddr) == SECT_FREE) {
			setSectData(saddr, SECT_LAST);
			new_saddr = saddr;
			break;
        }
		saddr++;
	} while(saddr < SECTOR_COUNT);

	if(!new_saddr) {
		file->errno = fsNoMoreSpace;
		return -1;
	}

	saddr = file->firstSect;

    if(!saddr) {
		file->firstSect = new_saddr;
		if(fsSeek(&metaFile, file->metadataOffset + sizeof(fsMetadata) - sizeof(SectAddr), SEEK_SET) != 0) {
			file->errno = fsFilesystemError;
			return -1;
		}
		fsWrite(&file->firstSect, 1, sizeof(SectAddr), &metaFile);
		if(metaFile.errno) {
			file->errno = metaFile.errno;
			return -1;
		}
		return fsSeek(file, 0, SEEK_SET);
	}

	//Locate last sector in file sector chain
    while(getSectData(saddr) > SECT_RESERVED)
		saddr = getSectData(saddr);

	//Append newly allocated sector
	setSectData(saddr, new_saddr);
	if(file->EOF) {
		file->EOF = FALSE;
		file->currentSect = new_saddr;
	}

    return 0;
}
//--------------------------------------------
#endif /* INCLUDE_NVFS */
